use crate::{allocator, array_list::LogicArrayList, ffi_util::import};

use super::{data::LogicSpellData, json::LogicJSONNode};

#[repr(C)]
pub struct LogicSpellDeck {
    vtable: usize,
    unk_gap_1: [u8; 28],
}

#[repr(C)]
pub struct LogicSpellCollection {
    pub spells: LogicArrayList<&'static mut LogicSpell>,
    pub current_sort: i32,
}

#[repr(C)]
pub struct LogicSpell {
    pub data: &'static LogicSpellData,
    pub level_index: i32,
    pub create_time: i32,
    pub count: i32,
    pub listener: &'static LogicSpellListener,
    pub new_upgrade: bool,
    pub new_flag: bool,
    pub new_count: i32,
    pub use_count: i32,
}

#[repr(C)]
pub struct LogicSpellListener {
    vtable: usize,
}

impl LogicSpell {
    pub fn new() -> &'static mut Self {
        import!(logic_spell_ctor(ptr: *mut LogicSpell) -> () = 0x117FCC + 1);
        let instance = allocator::new::<Self>();
        logic_spell_ctor(instance);
        unsafe { &mut *instance }
    }
}

impl LogicSpellDeck {
    #[expect(unused)]
    pub fn new() -> &'static mut Self {
        import!(logic_spell_deck_ctor(ptr: *mut LogicSpellDeck) -> () = 0x1188FC + 1);
        let instance = allocator::new::<Self>();
        logic_spell_deck_ctor(instance);
        unsafe { &mut *instance }
    }

    pub fn load(&mut self, data: &LogicJSONNode) {
        import!(logic_spell_deck_load(ptr: *mut LogicSpellDeck, data: *const LogicJSONNode) -> () = 0x118A7C + 1);
        logic_spell_deck_load(self, data);
    }

    pub fn clone(&self) -> &'static mut Self {
        import!(logic_spell_deck_clone(ptr: *const LogicSpellDeck) -> *mut LogicSpellDeck = 0x11890C + 1);
        unsafe { &mut *logic_spell_deck_clone(self) }
    }
}

impl LogicSpellCollection {
    pub fn add_spell(&mut self, spell: &'static LogicSpell) {
        import!(logic_spell_collection_add_spell(ptr: *const LogicSpellCollection, spell: *const LogicSpell) -> () = 0x118708 + 1);
        logic_spell_collection_add_spell(self, spell);
    }
}
