use crate::{
    allocator,
    ffi_util::{import, Nullable},
    logic::math::LogicLong,
    sc_string::ScString,
};

use super::PiranhaMessage;

#[repr(C)]
pub struct LoginMessage {
    pub parent: PiranhaMessage,
    pub account_id: &'static LogicLong,
    pub pass_token: Nullable<ScString>,
    pub client_major_version: i32,
    pub client_minor_version: i32,
    pub client_build: i32,
}

impl LoginMessage {
    pub const MESSAGE_TYPE: u16 = 10101;
}

#[repr(C)]
pub struct KeepAliveMessage {
    pub parent: PiranhaMessage,
}

impl KeepAliveMessage {
    pub const MESSAGE_TYPE: u16 = 10108;
}

#[repr(C)]
pub struct LoginOkMessage {
    pub parent: PiranhaMessage,
    pub account_id: &'static LogicLong,
    pub home_id: &'static LogicLong,
    pub pass_token: Nullable<ScString>,
    pub facebook_id: Nullable<ScString>,
    pub gamecenter_id: Nullable<ScString>,
    pub facebook_app_id: Nullable<ScString>,
    pub server_major_version: i32,
    pub server_minor_version: i32,
    pub server_build: i32,
    pub content_version: i32,
    pub environment: Nullable<ScString>,
    pub session_count: i32,
    pub play_time_seconds: i32,
    pub days_since_started_playing: i32,
    pub server_time: Nullable<ScString>,
    pub account_created_date: Nullable<ScString>,
    pub google_service_id: Nullable<ScString>,
    pub unk: i32,
}

impl LoginOkMessage {
    pub fn new() -> &'static mut Self {
        import!(login_ok_message_ctor(ptr: *mut LoginOkMessage) -> () = 0x12A644 + 1);
        let instance = allocator::new::<Self>();
        login_ok_message_ctor(instance);
        unsafe { &mut *instance }
    }
}

#[repr(C)]
pub struct KeepAliveServerMessage {
    pub parent: PiranhaMessage,
}

impl KeepAliveServerMessage {
    pub fn new() -> &'static mut Self {
        import!(keep_alive_server_message_ctor(ptr: *mut KeepAliveServerMessage) -> () = 0x129E7C + 1);
        let instance = allocator::new::<Self>();
        keep_alive_server_message_ctor(instance);
        unsafe { &mut *instance }
    }
}
