use crate::{
    allocator,
    array_list::LogicArrayList,
    ffi_util::{import, Nullable},
    logic::{
        avatar::LogicClientAvatar, command::LogicCommand, data::LogicNpcData, home::LogicClientHome,
    },
};

use super::PiranhaMessage;

#[repr(C)]
pub struct GoHomeMessage {
    pub parent: PiranhaMessage,
    unk_bool: bool,
}

impl GoHomeMessage {
    pub const MESSAGE_TYPE: u16 = 14101;
}

#[repr(C)]
pub struct EndClientTurnMessage {
    pub parent: PiranhaMessage,
    pub commands: &'static LogicArrayList<&'static LogicCommand>,
    pub tick: i32,
    pub checksum: i32,
}

impl EndClientTurnMessage {
    pub const MESSAGE_TYPE: u16 = 14102;
}

#[repr(C)]
pub struct StartMissionMessage {
    pub parent: PiranhaMessage,
    pub npc: &'static LogicNpcData,
}

impl StartMissionMessage {
    pub const MESSAGE_TYPE: u16 = 14104;
}

#[repr(C)]
pub struct OwnHomeDataMessage {
    pub parent: PiranhaMessage,
    pub logic_client_home: Nullable<LogicClientHome>,
    pub logic_client_avatar: Nullable<LogicClientAvatar>,
    pub random_seed: i32,
}

impl OwnHomeDataMessage {
    pub fn new() -> &'static mut Self {
        import!(own_home_data_message_ctor(ptr: *mut OwnHomeDataMessage) -> () = 0x130480 + 1);
        let instance = allocator::new::<Self>();
        own_home_data_message_ctor(instance);
        unsafe { &mut *instance }
    }
}

#[repr(C)]
pub struct OutOfSyncMessage {
    pub parent: PiranhaMessage,
    pub server_checksum: i32,
    pub client_checksum: i32,
    pub tick: i32,
}

impl OutOfSyncMessage {
    pub fn new() -> &'static mut Self {
        import!(out_of_sync_message_ctor(ptr: *mut OutOfSyncMessage) -> () = 0x130330 + 1);
        let instance = allocator::new::<Self>();
        out_of_sync_message_ctor(instance);
        unsafe { &mut *instance }
    }
}
