use tracing::info;

use crate::{
    logic::{
        math::LogicLong,
        LogicVersion,
    },
    network::message::{
        KeepAliveMessage, KeepAliveServerMessage, LoginMessage, LoginOkMessage,
    },
    sc_string::ScString,
    session::{PlayerSaveData, PlayerSession},
};

pub fn on_login_message(session: &mut PlayerSession, message: &LoginMessage) {
    info!(
        "LoginMessage received: account_id: {}, pass_token: {:?}, client version: {}.{}.{}",
        message.account_id,
        message.pass_token.get(),
        message.client_major_version,
        message.client_build,
        message.client_minor_version
    );

    let login_ok_message = LoginOkMessage::new();

    login_ok_message.account_id = LogicLong::new(0, 1);
    login_ok_message.home_id = LogicLong::new(0, 1);
    login_ok_message
        .pass_token
        .set(ScString::new("MostSecureTokenEver"));

    login_ok_message.server_major_version = LogicVersion::MAJOR;
    login_ok_message.server_build = LogicVersion::BUILD;
    login_ok_message.environment.set(ScString::new("dev"));

    let account_id = LogicLong {
        higher_int: 0,
        lower_int: 1,
    };

    session.player_save_data = Some(PlayerSaveData::create_default_data(&account_id));
    session.account_id = Some(account_id);

    session.send_message(login_ok_message);
    session.go_home();
}

pub fn on_keep_alive_message(session: &mut PlayerSession, _message: &KeepAliveMessage) {
    session.send_message(KeepAliveServerMessage::new());
}
