use crate::{
    logic::{avatar::LogicClientAvatar, home::LogicClientHome, mode::LogicGameMode},
    network::message::{OwnHomeDataMessage, SectorStateMessage},
};

pub fn build_own_home_data(
    logic_client_home: &LogicClientHome,
    logic_client_avatar: &LogicClientAvatar,
    random_seed: i32,
) -> &'static mut OwnHomeDataMessage {
    let own_home_data_message = OwnHomeDataMessage::new();

    own_home_data_message
        .logic_client_home
        .set(LogicClientHome::clone(logic_client_home));

    own_home_data_message
        .logic_client_avatar
        .set(LogicClientAvatar::clone(logic_client_avatar));

    own_home_data_message.random_seed = random_seed;
    own_home_data_message
}

pub fn build_sector_state(
    logic_game_mode: &LogicGameMode,
    client_avatar: &LogicClientAvatar,
) -> &'static mut SectorStateMessage {
    let sector_state_message = SectorStateMessage::new();
    sector_state_message.parent.stream.write_byte(0);
    sector_state_message.parent.stream.parent.checksum = 0;
    logic_game_mode.encode(
        &mut sector_state_message.parent.stream.parent,
        true,
        client_avatar,
        false,
    );

    sector_state_message
}
